@@ -55,7 +55,7 @@ pf(float myf)
 	uint32_t u;
     } fu;
     fu.f = myf;
-    sprintf(pf_str, "0x%x", fu.u);
+    snprintf(pf_str, sizeof(pf_str), "0x%x", fu.u);
     return pf_str;
 }
 
@@ -68,7 +68,7 @@ pd(double myd)
 	uint64_t u;
     } du;
     du.d = myd;
-    sprintf(pf_str, "0x%llx", (unsigned long long)du.u);
+    snprintf(pf_str, sizeof(pf_str), "0x%llx", (unsigned long long)du.u);
     return pf_str;
 }
 
@@ -240,7 +240,7 @@ main(int argc, char **argv)
 		printf("\t\t**** testing quantize algorithm %d...\n", quantize_mode[q]);
 		for (t = 0; t < NTYPES; t++)
 		{
-		    sprintf(file_name, "%s_quantize_%d_type_%d.nc", TEST, quantize_mode[q], xtype[t]);
+		    snprintf(file_name, sizeof(file_name), "%s_quantize_%d_type_%d.nc", TEST, quantize_mode[q], xtype[t]);
 #ifdef TESTNCZARR
 		    {
 			char url[NC_MAX_FILENAME + 1];
@@ -278,7 +278,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_1] = {1.1111111};
+		float float_data[DIM_LEN_1] = {1.1111111f};
 		double double_data[DIM_LEN_1] = {1.111111111111};
 
 		printf("\t\t**** testing quantize algorithm %d...\n", quantize_mode[q]);
@@ -309,8 +309,10 @@ main(int argc, char **argv)
 		    union FU fin;
 		    int nsd_att_in;
 		    union DU dfin;
-		    /* union FU fout; */
-		    /* union DU dfout; */
+		    union FU fout;
+		    union DU dfout;
+
+		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    /* Open the file and check metadata. */
 		    if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
@@ -349,15 +351,19 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, &float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, &double_in)) ERR;
-		    /* fout.f = float_data[0]; */
-		    /* dfout.d = double_data[0]; */
+#if 0
+		    fout.f = float_data[0];
+		    dfout.d = double_data[0];
+#endif
 		    fin.f = float_in;
 		    dfin.d = double_in;
-		    /* printf ("\nfloat_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-		    /*         float_data[0], fout.u, float_data[0], fin.u); */
-		    /* printf ("\ndouble_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n", */
-		    /*          double_data[0], dfout.u, double_data[0], dfin.u); */
+#if 0
+		    printf ("\nfloat_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+		            float_data[0], fout.u, float_data[0], fin.u);
+		    printf ("\ndouble_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n",
+		             double_data[0], dfout.u, double_data[0], dfin.u);
 
+#endif
 		    /* Check the results, slightly different for each quantize algorithm. */
 		    switch (quantize_mode[q])
 		    {
@@ -389,7 +395,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_1] = {1.1111111};
+		float float_data[DIM_LEN_1] = {1.1111111f};
 		double double_data[DIM_LEN_1] = {1.111111111111};
 
 		printf("\t\t**** testing quantize algorithm %d...\n", quantize_mode[q]);
@@ -420,8 +426,10 @@ main(int argc, char **argv)
 		    double double_in;
 		    union FU fin;
 		    union DU dfin;
-		    /* union FU fout; */
-		    /* union DU dfout; */
+		    union FU fout;
+		    union DU dfout;
+
+		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    /* Open the file and check metadata. */
 		    if (nc_open(FILE_NAME, NC_WRITE, &ncid)) ERR;
@@ -435,15 +443,18 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, &float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, &double_in)) ERR;
-		    /* fout.f = float_data[0]; */
-		    /* dfout.d = double_data[0]; */
+#if 0
+		    fout.f = float_data[0];
+		    dfout.d = double_data[0];
+#endif
 		    fin.f = float_in;
 		    dfin.d = double_in;
-		    /* printf ("\nfloat_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-		    /*         float_data[0], fout.u, float_data[0], fin.u); */
-		    /* printf ("\ndouble_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n", */
-		    /*         double_data[0], dfout.u, double_data[0], dfin.u); */
-
+#if 0
+		    printf ("\nfloat_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+		            float_data[0], fout.u, float_data[0], fin.u);
+		    printf ("\ndouble_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n",
+		            double_data[0], dfout.u, double_data[0], dfin.u);
+#endif
 		    /* Check the results, slightly different for each quantize algorithm. */
 		    switch (quantize_mode[q])
 		    {
@@ -475,7 +486,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_5] = {1.11111111, 1.0, 9.99999999, 12345.67, .1234567};
+		float float_data[DIM_LEN_5] = {1.11111111f, 1.0f, 9.99999999f, 12345.67f, .1234567f};
 		double double_data[DIM_LEN_5] = {1.1111111, 1.0, 9.999999999, 1234567890.12345, 123456789012345.0};
 		int x;
 
@@ -511,11 +522,12 @@ main(int argc, char **argv)
 		    };
 
 		    union FU fin;
-		    union FU fout;
 		    union FU xpect[NUM_QUANTIZE_MODES][DIM_LEN_5];
 		    union DU dfin;
+		    union FU fout;
 		    union DU dfout;
 		    union DU double_xpect[NUM_QUANTIZE_MODES][DIM_LEN_5];
+
 		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    switch (quantize_mode[q])
@@ -572,18 +584,24 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, double_in)) ERR;
-		    /* printf("\n"); */
+#if 0
+		    printf("\n");
+#endif
 		    for (x = 0; x < DIM_LEN_5; x++)
 		    {
-			fout.f = float_data[x];
 			fin.f = float_in[x];
-			/* printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-			/*         float_data[x], fout.u, float_data[x], fin.u); */
+#if 0
+			fout.f = float_data[x];
+			printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+			        float_data[x], fout.u, float_data[x], fin.u);
+#endif
 			if (fin.u != xpect[q][x].u) ERR;
-			dfout.d = double_data[x];
 			dfin.d = double_in[x];
-			/* printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n", */
-			/* 		       double_data[x], dfout.u, double_data[x], dfin.u); */
+#if 0
+			dfout.d = double_data[x];
+			printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n",
+			 		       double_data[x], dfout.u, double_data[x], dfin.u);
+#endif
 			if (dfin.u != double_xpect[q][x].u) ERR;
 		    }
 
@@ -599,7 +617,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_1] = {1.1111111};
+		float float_data[DIM_LEN_1] = {1.1111111f};
 		double double_data[DIM_LEN_1] = {1.111111111111};
 
 		printf("\t\t**** testing quantize algorithm %d...\n", quantize_mode[q]);
@@ -635,6 +653,7 @@ main(int argc, char **argv)
 		    union FU fout;
 		    union DU dfout;
 		    int nsd_att_in;
+
 		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    /* Open the file and check metadata. */
@@ -674,16 +693,17 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, &float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, &double_in)) ERR;
-		    fout.f = (float)double_data[0];
 		    fin.f = float_in;
-		    dfout.d = float_data[0];
 		    dfin.d = double_in;
-		    /* printf ("\ndouble_data: %15g   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-		    /*         double_data[0], fout.u, float_in, fin.u); */
-		    /* printf ("\nfloat_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n", */
-		    /*         float_data[0], dfout.u, double_in, dfin.u); */
+#if 0
+		    fout.f = (float)double_data[0];
+		    dfout.d = float_data[0];
+		    printf ("\ndouble_data: %15g   : 0x%x  float_data_in: %10f   : 0x%x\n",
+		           double_data[0], fout.u, float_in, fin.u);
+		    printf ("\nfloat_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%lx\n",
+		            float_data[0], dfout.u, double_in, dfin.u);
 
-		    
+#endif		    
 		    switch (quantize_mode[q])
 		    {
 		    case NC_QUANTIZE_BITGROOM:
@@ -714,7 +734,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_5] = {1.11111111, 1.0, 9.99999999, 12345.67, .1234567};
+		float float_data[DIM_LEN_5] = {1.11111111f, 1.0, 9.99999999f, 12345.67f, .1234567f};
 		double double_data[DIM_LEN_5] = {1.1111111, 1.0, 9.999999999, 1234567890.12345, 123456789012345.0};
 		int x;
 
@@ -755,6 +775,7 @@ main(int argc, char **argv)
 		    union FU fout;
 		    union DU dfout;
 		    union DU double_xpect[NUM_QUANTIZE_MODES][DIM_LEN_5];
+
 		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    switch (quantize_mode[q])
@@ -811,17 +832,21 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, double_in)) ERR;
-		    /* printf("\n"); */
+#if 0
+		    printf("\n");
+#endif
 		    for (x = 0; x < DIM_LEN_5; x++)
 		    {
-			fout.f = float_data[x];
 			fin.f = float_in[x];
-			/* printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-			/*         float_data[x], fout.u, float_data[x], fin.u); */
-			dfout.d = double_data[x];
 			dfin.d = double_in[x];
-			/* printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n", */
-			/*        double_data[x], dfout.u, double_data[x], dfin.u); */
+#if 0
+			fout.f = float_data[x];
+			printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+			        float_data[x], fout.u, float_data[x], fin.u);
+			dfout.d = double_data[x];
+			printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n",
+			       double_data[x], dfout.u, double_data[x], dfin.u);
+#endif
 			if (fin.u != xpect[q][x].u) ERR;
 			if (dfin.u != double_xpect[q][x].u) ERR;
 		    }
@@ -838,7 +863,7 @@ main(int argc, char **argv)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_5] = {1.11111111, NC_FILL_FLOAT, 9.99999999, 12345.67, NC_FILL_FLOAT};
+		float float_data[DIM_LEN_5] = {1.11111111f, NC_FILL_FLOAT, 9.99999999f, 12345.67f, NC_FILL_FLOAT};
 		double double_data[DIM_LEN_5] = {1.1111111, NC_FILL_DOUBLE, 9.999999999, 1234567890.12345, NC_FILL_DOUBLE};
 		int x;
 
@@ -879,6 +904,7 @@ main(int argc, char **argv)
 		    union FU fout;
 		    union DU dfout;
 		    union DU double_xpect[NUM_QUANTIZE_MODES][DIM_LEN_5];
+
 		    NC_UNUSED(fout); NC_UNUSED(dfout);
 
 		    switch (quantize_mode[q])
@@ -935,17 +961,21 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, double_in)) ERR;
-		    /* printf("\n"); */
+#if 0
+		    printf("\n");
+#endif
 		    for (x = 0; x < DIM_LEN_5; x++)
 		    {
-			fout.f = float_data[x];
 			fin.f = float_in[x];
-			dfout.d = double_data[x];
 			dfin.d = double_in[x];
-			/* printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-			/*         float_data[x], fout.u, float_data[x], fin.u); */
-			/* printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n", */
-			/*        double_data[x], dfout.u, double_data[x], dfin.u); */
+#if 0
+			fout.f = float_data[x];
+			dfout.d = double_data[x];
+			printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+			        float_data[x], fout.u, float_data[x], fin.u);
+			printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n",
+			       double_data[x], dfout.u, double_data[x], dfin.u);
+#endif
 			if (fin.u != xpect[q][x].u) ERR;
 			if (dfin.u != double_xpect[q][x].u) ERR;
 		    }
@@ -958,13 +988,13 @@ main(int argc, char **argv)
 	SUMMARIZE_ERR;
 	printf("\t**** testing more quantization values with custom fill values...\n");
 	{
-#define CUSTOM_FILL_FLOAT 99.99999
+#define CUSTOM_FILL_FLOAT 99.99999f
 #define CUSTOM_FILL_DOUBLE -99999.99999
 	    for (q = 0; q < NUM_QUANTIZE_MODES; q++)
 	    {
 		int ncid, dimid, varid1, varid2;
 		int quantize_mode_in, nsd_in;
-		float float_data[DIM_LEN_5] = {1.11111111, CUSTOM_FILL_FLOAT, 9.99999999, 12345.67, CUSTOM_FILL_FLOAT};
+		float float_data[DIM_LEN_5] = {1.11111111f, CUSTOM_FILL_FLOAT, 9.99999999f, 12345.67f, CUSTOM_FILL_FLOAT};
 		double double_data[DIM_LEN_5] = {1.1111111, CUSTOM_FILL_DOUBLE, 9.999999999, 1234567890.12345, CUSTOM_FILL_DOUBLE};
 		float custom_fill_float = CUSTOM_FILL_FLOAT;
 		double custom_fill_double = CUSTOM_FILL_DOUBLE;
@@ -976,9 +1006,9 @@ main(int argc, char **argv)
 		if (nc_create(FILE_NAME, mode, &ncid)) ERR;
 		if (nc_def_dim(ncid, DIM_NAME_1, DIM_LEN_5, &dimid)) ERR;
 		if (nc_def_var(ncid, VAR_NAME_1, NC_FLOAT, NDIM1, &dimid, &varid1)) ERR;
-		if (nc_put_att_float(ncid, varid1, _FillValue, NC_FLOAT, 1, &custom_fill_float)) ERR;
+		if (nc_put_att_float(ncid, varid1, NC_FillValue, NC_FLOAT, 1, &custom_fill_float)) ERR;
 		if (nc_def_var(ncid, VAR_NAME_2, NC_DOUBLE, NDIM1, &dimid, &varid2)) ERR;
-		if (nc_put_att_double(ncid, varid2, _FillValue, NC_DOUBLE, 1, &custom_fill_double)) ERR;
+		if (nc_put_att_double(ncid, varid2, NC_FillValue, NC_DOUBLE, 1, &custom_fill_double)) ERR;
 
 		/* Turn on quantize for both vars. */
 		if (nc_def_var_quantize(ncid, varid1, quantize_mode[q], NSD_3)) ERR;
@@ -1009,6 +1039,7 @@ main(int argc, char **argv)
 		    union FU fout;
 		    union DU dfout;
 		    union DU double_xpect[NUM_QUANTIZE_MODES][DIM_LEN_5];
+
 		    NC_UNUSED(fout); NC_UNUSED(dfout);
 		    
 		    switch (quantize_mode[q])
@@ -1065,17 +1096,21 @@ main(int argc, char **argv)
 		    /* Check the data. */
 		    if (nc_get_var(ncid, varid1, float_in)) ERR;
 		    if (nc_get_var(ncid, varid2, double_in)) ERR;
-		    /* printf("\n"); */
+#if 0
+		    printf("\n");
+#endif
 		    for (x = 0; x < DIM_LEN_5; x++)
 		    {
-			fout.f = float_data[x];
 			fin.f = float_in[x];
-			dfout.d = double_data[x];
 			dfin.d = double_in[x];
-			/* printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n", */
-			/*         float_data[x], fout.u, float_data[x], fin.u); */
-			/* printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n", */
-			/*        double_data[x], dfout.u, double_data[x], dfin.u); */
+#if 0
+			fout.f = float_data[x];
+			dfout.d = double_data[x];
+			printf ("float_data: %10f   : 0x%x  float_data_in: %10f   : 0x%x\n",
+			        float_data[x], fout.u, float_data[x], fin.u);
+			printf("double_data: %15g   : 0x%16lx  double_data_in: %15g   : 0x%16lx\n",
+			       double_data[x], dfout.u, double_data[x], dfin.u);
+#endif
 			if (fin.u != xpect[q][x].u) ERR;
 			if (dfin.u != double_xpect[q][x].u) ERR;
 		    }
@@ -1243,10 +1278,12 @@ main(int argc, char **argv)
 			union DU dfin;
 			fin.f = float_data_in[x];
 			dfin.d = double_data_in[x];
-			/* printf ("%d float_data_in : %08.8f   : 0x%x expected %08.8f   : 0x%x\n",  */
-			/*  	x, float_data_in[x], fin.u, xpect[q][x].f, xpect[q][x].u);  */
-			/* printf ("%d double_data_in : %15g   : 0x%lx expected %15g   : 0x%lx\n", */
-			/* 	x, double_data_in[x], dfin.u, double_xpect[q][x].d, double_xpect[q][x].u); */
+#if 0
+			printf ("%d float_data_in : %08.8f   : 0x%x expected %08.8f   : 0x%x\n",
+			 	x, float_data_in[x], fin.u, xpect[q][x].f, xpect[q][x].u);
+			printf ("%d double_data_in : %15g   : 0x%lx expected %15g   : 0x%lx\n",
+				x, double_data_in[x], dfin.u, double_xpect[q][x].d, double_xpect[q][x].u);
+#endif
 			if (fin.u != xpect[q][x].u) ERR;
 			if (dfin.u != double_xpect[q][x].u) ERR;
 		    }
@@ -1275,8 +1312,8 @@ main(int argc, char **argv)
 		ERR;
 	    for (i = 0; i < DIM_LEN_SIMPLE; i++)
 	    {
-		float_data[i] = 1.5 * i;
-		double_data[i] = 1.5 * i;
+		float_data[i] = 1.5f * (float)i;
+		double_data[i] = 1.5 * (double)i;
 	    }
 
 	    /* Create the file. */
@@ -1297,7 +1334,7 @@ main(int argc, char **argv)
 	    if (nc_def_var_quantize(ncid, varid2, NC_QUANTIZE_BITGROOM, NSD_3)) ERR;
 
 #ifdef TESTNCZARR
-#ifdef ENABLE_NCZARR_FILTERS
+#ifdef NETCDF_ENABLE_NCZARR_FILTERS
 	    /* Set up zlib compression. This will work better because the
 	     * data are quantized, yielding a smaller output file. We will
 	     * set compression level to 1, which is usually the best
@@ -1332,7 +1369,7 @@ main(int argc, char **argv)
 
 		for (i = 0; i < DIM_LEN_SIMPLE; i++)
 		{
-		    if (fabs(float_data_in[i] - float_data[i]) > EPSILON)
+		    if (fabsf(float_data_in[i] - float_data[i]) > EPSILON)
 			ERR;
 		    if (fabs(double_data_in[i] - double_data[i]) > EPSILON)
 			ERR;
